/*******************************************************************************
 * Copyright (c) 2010- Association for Decentralized Information Management in
 * Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.accessor.impl;

import java.util.concurrent.locks.Lock;

import org.simantics.databoard.accessor.emit.EventEmitter;
import org.simantics.databoard.accessor.emit.ImmediateEmitter;

/**
 * This class carries implementation specific accessor parameters.
 * The object is typically propagated to sub-accessors.
 *
 * @author Toni Kalajainen <toni.kalajainen@vtt.fi>
 */
public class AccessorParams {
	
	public static final AccessorParams DEFAULT = new AccessorParams(null, null, ImmediateEmitter.INSTANCE, false);
	
	/*
	 * With locks there are three use scenarios:
	 * 
	 *  a) No locks
	 *  b) One lock, mutual exclusion
	 *  c) Read and write lock, See ReentrantReadWriteLock
	 * 
	 */
	
	/** Read Lock, optional */
	public final Lock readLock;
	/** Write Lock, optional */
	public final Lock writeLock;

	/** Event Emitter */
	public EventEmitter emitter;

	/** 
	 * Type adapt arguments. An example, if this setting is true, the following would work:  
	 *   Float f = DoubleAccessor.getValue(floatBinding)
	 */
	public final boolean typeAdapter;

	public AccessorParams(Lock readLock, Lock writeLock, EventEmitter emitter, boolean typeAdapter) {
		this.readLock = readLock;
		this.writeLock = writeLock;
		this.emitter = emitter;
		this.typeAdapter = typeAdapter;
	}
	
}

