package org.simantics.databoard.channel;

import java.util.Collection;

import org.simantics.databoard.Accessors;
import org.simantics.databoard.accessor.Accessor;
import org.simantics.databoard.binding.Binding;
import org.simantics.databoard.binding.mutable.Variant;

/**
 * Client interface for command channel. <p>
 *  
 * Command channel is a request-response channel. 
 * All operations are asynchronous. <p>
 * 
 * For each callable command there is a commandId and specified datatype for request and error.
 *
 * @author Toni Kalajainen <toni.kalajainen@iki.fi>
 */
public interface CommandChannel {
	
	/**
	 * Invoke a service request. <p> 
	 * 
	 * Note it is a good idea create accessor with
	 * {@link Accessors#getAccessor(Binding, Object)}. <p>
	 * 
	 * @param commandId command identifier
	 * @param request the request object
	 * @return asynchrouns result object
	 */
	AsyncRequest invoke( Variant commandId, Accessor request );
	
	/**
	 * Get all callable commands of the channel.
	 * This result is list of command specifications. For each command there is
	 * unique identification, result type and error type.  
	 * 
	 * @return commands
	 */
	Collection<CommandSpec> getCommands();
	
}
