/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.type;

import java.util.Set;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import org.simantics.databoard.accessor.reference.ChildReference;
import org.simantics.databoard.annotations.Optional;
import org.simantics.databoard.util.IdentityPair;
import org.simantics.databoard.util.Limit;
import org.simantics.databoard.util.ObjectUtils;
import org.simantics.databoard.util.Range;

public class StringType extends Datatype {
		
    /** Mime-Type */    
    public @Optional String pattern;
    public @Optional String mimeType;         
    public @Optional String length;
    transient Range _length;
    
    // Compiled pattern
    private transient Pattern pattern_;
    
    public StringType () {}
    
    public StringType (String pattern) {
    	this.pattern = pattern;
    }

    public StringType (String pattern, String mimeType, Range length) {
    	this.pattern = pattern;
    	this.mimeType = mimeType;
    	setLength(length);
    }

    public StringType (String pattern, String mimeType, String length) {
    	this.pattern = pattern;
    	this.mimeType = mimeType;
    	setLength(length);
    }
    
    @Override
    public int count() {
    	return 0;
    }
    
    @Override
    public Datatype getComponentType(int index) {
    	throw new IllegalArgumentException();
    }
    
    @Override
    public Datatype getComponentType(ChildReference path) {
    	if (path==null) return this;
    	throw new IllegalArgumentException();
    }    
    
    @Override
    protected boolean deepEquals(Object obj,
    		Set<IdentityPair<Datatype, Datatype>> compareHistory) {
		if (this==obj) return true;
		if (obj instanceof StringType == false) return false;
		StringType other = (StringType) obj;
		return 
		    ObjectUtils.objectEquals(other.pattern, pattern) &&
			ObjectUtils.objectEquals(other.mimeType, mimeType) &&
			ObjectUtils.objectEquals(other.length, length);
	}
    
	@Override
	public int hashCode() {		
		return 0x43676323 + 
		ObjectUtils.hashCode(pattern)*27+
		ObjectUtils.hashCode(mimeType)*13+
		ObjectUtils.hashCode(length)*17;
	}    

	@Override
	public void accept(Visitor1 v, Object obj) {
	    v.visit(this, obj);        
	}

	@Override
	public <T> T accept(Visitor<T> v) {
	    return v.visit(this);
	}

	public String getPattern() {
		return pattern;
	}

	public void setPattern(String pattern) 
	throws PatternSyntaxException 
	{
		this.pattern = pattern;
		this.pattern_ = pattern==null ? null : Pattern.compile(pattern);
	}

	public String getMimeType() {
		return mimeType;
	}

	public void setMimeType(String mimeType) {
		this.mimeType = mimeType;
	}

	public Range getLength() {
		return _length;
	}

	public int minLength() {
    	if (_length==null) return 0;
    	Limit l = _length.getLower();
    	int value = l.getValue().intValue();
    	if (l.isExclusive()) value++;
    	return value;
	}
	
	public int maxLength() {
    	if (_length==null) return Integer.MAX_VALUE;
    	Limit l = _length.getUpper();
    	int value = l.getValue().intValue();
    	if (l.isExclusive()) value--;
    	return value;
	}

	/**
	 * Get compiled reg exp pattern 
	 * 
	 * @return pattern or null
	 */
	public Pattern getCompiledPattern()
	{
		return pattern_;
	}

	public void setLength(String length) {
    	this.length = length;
    	this._length = length == null ? null : Range.valueOfUnchecked(length);
	}
	
	public void setLength(Range range) {
		this._length = range;
		this.length = range == null ? null : range.toString();
	}
	
}
